@@ -1,4 +1,4 @@
-use dovepipe::{Source, send_file};
+use dovepipe::{send_file, Source};
 
 #[tokio::main]
 async fn main() {
